/****************************************************************************/
/*	Purpose:	Driver for LCD Display 16x4									*/
/*	Version:	1.0															*/
/*	Author:		Mike Schaub													*/
/*	Date:		27.09.2005													*/
/*	www:		www.microcontroller-starterkits.de							*/
/*	email:		mike@microcontroller-starterkits.de							*/
/*	Software:	WinARM to compile											*/
/*	Hardware:	LPC2194 with ARMEvoBoard									*/
/****************************************************************************/

#include "lpc21xx_keil.h"
//	#include "GLCD_128x64.h"
#include <symbols16x16.h>
#include <font5x8.h>
//	#include <math.h> 


#define  GLCD_D0	0x00000001		//	P0.00
#define  GLCD_D1	0x00000002		//	P0.01
#define  GLCD_D2	0x00000004		//	P0.02
#define  GLCD_D3	0x00000008		//	P0.03
#define  GLCD_D4	0x00000010		//	P0.04
#define  GLCD_D5	0x00000020		//	P0.05
#define  GLCD_D6	0x00000040		//	P0.06
#define  GLCD_D7	0x00000080		//	P0.07

#define  GLCD_RS	0x01000000		//	P1.24
#define  GLCD_RW	0x02000000		//	P1.25
#define  GLCD_EN	0x04000000		//	P1.26
#define  GLCD_CS1	0x08000000		//	P1.27
#define  GLCD_CS2	0x10000000		//	P1.28
#define  GLCD_RES	0x40000000		//	P1.30

#define  GLCD_DATA	0x000000FF
#define  GLCD_CTRL	(GLCD_EN|GLCD_RS|GLCD_RW|GLCD_CS1|GLCD_CS2|GLCD_RES)


#define INST			0x00	//	Register select	: 0=INSTRUCTION-Reg.
#define DATA			GLCD_RS	//	Register select	: 1=DATA-Reg.
#define WRITE			0x00	//	ReadWrite	: 0=write
#define READ			GLCD_RW	//	ReadWrite	: 2=read

#define ON				0x00000001
#define OFF				0x00000000

#define RESET			0x00000010	//	Reset		: 0=active
#define BUSY			0x00000080	//	Busy		: 1=active
#define DISP_ON_OFF		0x0000003E
#define DISP_SET_Y		0x00000040
#define DISP_SET_X		0x000000B8
#define DISP_START_LINE	0x000000C0

#define CLEAR			0x00
#define BLACK			0xFF

#define	PASS1			0x00000100
#define	PASS2			0x00000200
#define	PASS3			0x00000400
#define	PASS4			0x00000800


unsigned int	chip_nr = GLCD_CS1;
unsigned char	global_x,global_y = 0;



void wait(unsigned int zeit);


void wait_on_reset(void);
void wait_if_busy(void);
void display(unsigned long status);
void start_line(unsigned char spos);
void set_page(unsigned char xadr);
void set_x_adr(unsigned char yadr);
void write_lcd_data(unsigned char lcddata);
unsigned char read_lcd_data(void);
void set_dot(unsigned char x, unsigned char y);
void clr_dot(unsigned char x, unsigned char y);
void lcd_background(unsigned char	bg);

void line(unsigned char status, unsigned char xpos1, unsigned char ypos1, unsigned char xpos2, unsigned char ypos2);
void rectangle(unsigned char status, unsigned char xpos1, unsigned char ypos1, unsigned char xpos2, unsigned char ypos2);
void put_ch(unsigned char ch);
void put_text_xy(unsigned char x, unsigned char y, const unsigned char * s);




void wait(unsigned int zeit)
{
	while (zeit) zeit--;
}


void wait_on_reset(void)
{
	unsigned long	reset;		//	resetflag

	IOCLR0 |= GLCD_DATA;
	IOCLR1 |= GLCD_CTRL;
	IOSET1 |= (READ | chip_nr | GLCD_RES);
	IODIR0 &= ~(GLCD_DATA);	//	define LCD data pins as inputs
	IOSET1 |= GLCD_EN;

	do
	{
		reset = IOPIN0 & RESET;
		wait(100);
	}
	while (reset);

	IOCLR1 |= (GLCD_EN | READ | chip_nr);
	IODIR0 |= GLCD_DATA;		//	define LCD data pins as outputs

	IODIR0 |= (PASS1 | PASS2 | PASS3 | PASS4);
	IOSET0 |= PASS1;
	
	wait(100);
}


void wait_if_busy(void)
{
	unsigned long	bf;			//	busyflag

	IOSET1 |= (READ | chip_nr | GLCD_RES);
	IODIR0 &= ~(GLCD_DATA);	//	define LCD data pins as inputs
	IOSET1 |= GLCD_EN;

	do
	{
		bf = IOPIN0 & BUSY;
		wait(100);
	}
	while (bf == 0x80);

	IOCLR1 |= (GLCD_EN | READ | chip_nr);
	IODIR0 |= GLCD_DATA;		//	define LCD data pins as outputs

	IOSET0 |= PASS2;
}


void display(unsigned long status)
{
	unsigned int	z;

	for (z=0;z<=1;z++)
	{
		if (z == 0) chip_nr = GLCD_CS1;
		else chip_nr = GLCD_CS2;
		wait_if_busy();	
		IOSET1 |= (INST | chip_nr | GLCD_RES);
		IOCLR0 |= GLCD_DATA;
		IOSET0 |= (DISP_ON_OFF | status);
		
		wait(10000);
		
		IOSET1 = GLCD_EN;
		wait(10000);
		IOCLR1 = GLCD_EN;
		wait(10000);

		IOCLR1 |= chip_nr;
	}
	IOSET0 |= PASS3;
}


void start_line(unsigned char spos)		//	(0...63)
{
	wait_if_busy();	
	IOSET1 |= (INST | chip_nr | GLCD_RES);
	IOCLR0 |= GLCD_DATA;
	IOSET0 |= (DISP_START_LINE | spos);

	IOSET1 = GLCD_EN;
	wait(100);
	IOCLR1 = GLCD_EN;
	wait(100);

	IOCLR1 |= chip_nr;
}


void set_page(unsigned char xadr)		//	(0...7)
{
	wait_if_busy();	
	IOSET1 |= (INST | chip_nr | GLCD_RES);
	IOCLR0 |= GLCD_DATA;
	IOSET0 |= (DISP_SET_X | xadr);

	IOSET1 = GLCD_EN;
	wait(100);
	IOCLR1 = GLCD_EN;
	wait(100);
	
	IOCLR1 |= chip_nr;
}


void set_x_adr(unsigned char yadr)		//	(0...63)
{
	wait_if_busy();	
	IOSET1 |= (INST | chip_nr | GLCD_RES);
	IOCLR0 |= GLCD_DATA;
	IOSET0 |= (DISP_SET_Y | yadr);

	IOSET1 = GLCD_EN;
	wait(10000);
	IOCLR1 = GLCD_EN;
	wait(10000);

	IOCLR1 |= chip_nr;
}


void write_lcd_data(unsigned char lcddata)
{
	wait_if_busy();	
	IOSET1 |= (DATA | chip_nr | GLCD_RES);
	IOCLR0 |= GLCD_DATA;
	IOSET0 |= lcddata;

	IOSET1 = GLCD_EN;
	wait(1);
	IOCLR1 = GLCD_EN;
	wait(1);
	
	IOCLR1 |= DATA | chip_nr;
}


unsigned char read_lcd_data(void)
{
	unsigned char lcd_data;

	wait_if_busy();	
	IODIR0 &= ~(GLCD_DATA);	//	define LCD data pins as inputs
	IOSET1 |= (READ |DATA | chip_nr | GLCD_RES);
	wait(1);
	IOSET1 |= GLCD_EN;
	wait(1);
	lcd_data = IOPIN0 & 0x000000FF;

	IOCLR1 = GLCD_EN;
	IODIR1 |= GLCD_DATA;		//	define LCD data pins as outputs
	IOCLR1 |= DATA;

	return(lcd_data);
}


void set_dot(unsigned char x, unsigned char y)
{
	unsigned char	temp;
	unsigned char	xpos;
	unsigned char	ypos;

	if (x & 0x40)
	{
		chip_nr = GLCD_CS2;
		xpos = x & 0x3F;
	}
	else
	{
		chip_nr = GLCD_CS1;
		xpos = x;
	}

	ypos = y / 8;
	set_page(ypos);
	set_x_adr(xpos);
	temp = read_lcd_data();		//	dummy read
	temp = read_lcd_data();

	set_page(ypos);
	set_x_adr(xpos);
	ypos = y % 8;
	write_lcd_data(temp | (1 << ypos));
}


void clr_dot(unsigned char x, unsigned char y)
{
	unsigned char	temp;
	unsigned char	xpos;
	unsigned char	ypos;

	if (x & 0x40)
	{
		chip_nr = GLCD_CS2;
		xpos = x & 0x3F;
	}
	else
	{
		chip_nr = GLCD_CS1;
		xpos = x;
	}

	ypos = y / 8;
	set_page(ypos);
	set_x_adr(xpos);
	temp = read_lcd_data();		//	dummy read
	temp = read_lcd_data();

	set_page(ypos);
	set_x_adr(xpos);
	ypos = y % 8;
	temp &= ~(1 << (ypos));
	write_lcd_data(temp);
}


void lcd_background(unsigned char	bg)
{
	unsigned int	i,j;

	chip_nr = GLCD_CS1;
	start_line(0);
	set_page(0);
	set_x_adr(0);

	for(i=0;i<=7;i++)
	{
		set_page(i);
		for(j=0;j<=63;j++) write_lcd_data(bg);
	}

	chip_nr = GLCD_CS2;
	start_line(0);
	set_page(0);
	set_x_adr(0);

	for(i=0;i<=7;i++)
	{
		set_page(i);
		for(j=0;j<=63;j++) write_lcd_data(bg);
	}
	chip_nr = GLCD_CS1;
}


  
int main(void)
{
	unsigned char	i;

	MAMCR = 2;									//	MAM functions fully enabled

	IODIR1 |= GLCD_CTRL;			//	define LCD-Pins as outputs
	IOCLR1 |= GLCD_CTRL;

	IODIR0 |= (PASS1 | PASS2 | PASS3 | PASS4);
	IOSET0 |= PASS1;

	wait(50000);

	wait_on_reset();
	lcd_background(CLEAR);
	display(ON);
	set_page(0);
	set_x_adr(0);

	for(i=0;i<63;i++) set_dot(i,i);

	line(1,0,0,127,63);
	
	put_text_xy(1,1,"Hallo Du!");
	put_text_xy(1,11,"Hier Ich!");
	
	while (1)	
	{
	}
	
	return 0; // never reached
}


void line(unsigned char status, unsigned char xpos1, unsigned char ypos1, unsigned char xpos2, unsigned char ypos2)
{
	unsigned char	xpos,ypos;
	unsigned char	x1,x2,y1,y2;
	
	float	dx,dy,adx,ady;
	float	d,m;

	d = 0;
	m = 0;

	if (xpos1 < xpos2)
	{
		x1 = xpos1;
		x2 = xpos2;
		y1 = ypos1;
		y2 = ypos2;
	}
	else
	{
		x1 = xpos2;
		x2 = xpos1;
		y1 = ypos2;
		y2 = ypos1;
	}

	dx = (float) (x2 - x1);
	dy = (float) (y2 - y1);
	
	
	adx = (dx);
	ady = (dy);

	d = 0;

	if (adx >= ady)
	{
		if (dx == 0) m = 1;
		else m = dy/dx;
		ypos = y1;
		for(xpos=x1;xpos<=x2;xpos++)
		{
			if (m>=0)
			{	
				if(d>=m)
				{
					ypos++;
					d-=1;
				}
			}
			else
			{	
				if(d<=-m)
				{
					ypos--;
					d+=1;
				}
			}
			if(status) set_dot(xpos, ypos);
			else clr_dot(xpos, ypos);
			d+=m;
		}
	}

	else
	{
		if (dy == 0) m = 1;
		else m = dx/dy;
		xpos = x1;
		for(ypos=y1;ypos<=y2;ypos++)
		{
			if (m>=0)
			{
				if(d>=m)
				{
					xpos++;
					d-=1;
				}
			}
			else
			{
				if(d<=-m)
				{
					xpos--;
					d+=1;
				}
			}
			if(status) set_dot(xpos, ypos);
			else clr_dot(xpos, ypos);
			d += m;
		}
	}

}


void rectangle(unsigned char status, unsigned char xpos1, unsigned char ypos1, unsigned char xpos2, unsigned char ypos2)
{
	unsigned char	z;
	
	for (z=xpos1;z<=xpos2;z++)
	{
		if (status)
		{
			set_dot(z,ypos2);
			set_dot(z,ypos1);
		}
		else
		{
			clr_dot(z,ypos1);
			clr_dot(z,ypos2);
		}
	}
	for (z=ypos1;z<=ypos2;z++)
	{
		if (status)
		{
			set_dot(xpos1,z);
			set_dot(xpos2,z);
		}
		else
		{
			clr_dot(xpos1,z);
			clr_dot(xpos2,z);
		}
	}
}


void put_ch(unsigned char ch)
{
	unsigned char	i;
	
	for(i=0;i<=4;i++)
	{
		if (global_x & 0x80)
		{
			global_x = 0;
			global_y++;
			chip_nr = GLCD_CS1;
			set_page(global_y);
			chip_nr = GLCD_CS2;
			set_page(global_y);
		}
		if (global_y & 0x08) global_y = 0;
		if (global_x & 0x40) chip_nr = GLCD_CS2;
		else chip_nr = GLCD_CS1;
		
		write_lcd_data(font5x8[((ch - 32) * 5) + i]);
		wait(5);
	}
	write_lcd_data(0);
}


void put_text_xy(unsigned char x, unsigned char y, const unsigned char * s)
{
	unsigned char	j,xpos,ypos;
	unsigned long	z;

	if (y & 0x40)
	{
		z = GLCD_CS2;
		ypos = y & 0x3F;
	}
	else
	{
		z = GLCD_CS1;
		ypos = y;
	}

	xpos = x / 8;
	global_x = x;
	global_y = ypos;

	chip_nr = GLCD_CS1;
	set_page(xpos);
	set_x_adr(ypos);
	chip_nr = GLCD_CS2;
	set_page(xpos);
	set_x_adr(ypos);

	chip_nr = z;
	
	j = 0;

	do
	{
		put_ch(s[j]);
		j++;
	}
	while (s[j] != '\0');
}

